/*
 * Copyright (c) 2013, TRIOLOGY GmbH
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * https://www.scm-manager.com
 *
 */

Ext.ns('Triology');
Ext.ns('Triology.groupmanager');

Triology.groupmanager.I18n = {
  groupManagerText : 'Group Manager',
  emptyText : 'Select a group.',
  removeTitleText : 'Remove Group',
  removeMsgText : 'Remove Group "{0}"?',
  errorTitleText : 'Error',
  errorMsgText : 'Group deletion failed',
  colNameText : 'Name',
  colDescriptionText : 'Description',
  colMembersText : 'Members',
  colGroupManagerText : 'Group-Manager',
  colCreationDateText : 'Creation date',
  colTypeText : 'Type',
  emptygroupManagerStoreText : 'There are no groups configured you can manage.',
  groupFormTitleText : 'Group Form',
  settingsText : 'Settings',
  updateErrorMsgText : 'Group update failed',
  createErrorMsgText : 'Group creation failed',
  groupManagerHelpText : 'Usernames of the Group-Managers.',
  groupManagerFormTitleText : 'Group Manager Form',
  sectionSecurityText : 'Security'

};

// German translation

if ('de' == i18n.country) {
  Triology.groupmanager.I18n = {
    groupManagerText : 'Gruppen-Manager',
    emptyText : 'Wählen Sie eine Gruppe aus.',
    removeTitleText : 'Gruppe entfernen',
    removeMsgText : 'Gruppe "{0}" entfernen?',
    errorTitleText : 'Fehler',
    errorMsgText : 'Das Löschen der Gruppe ist fehlgeschlagen!',
    colNameText : 'Name',
    colDescriptionText : 'Beschreibung',
    colMembersText : 'Mitglieder',
    colGroupManagerText : 'Gruppen-Manager',
    colCreationDateText : 'Erstellungsdatum',
    colTypeText : 'Typ',
    emptygroupManagerStoreText : 'Es sind keine Gruppen konfiguriert, die Sie verwalten können.',
    groupFormTitleText : 'Gruppen Formular',
    settingsText : 'Einstellungen',
    updateErrorMsgText : 'Update der Gruppen-Manager fehlgeschalgen!',
    createErrorMsgText : 'Erstellung des Gruppen-Managers fehlgeschlagen!',
    groupManagerHelpText : 'Die Benutzernamen der Gruppen-Manager.',
    groupManagerFormTitleText : 'Gruppen-Manager Formular',
    sectionSecurityText : 'Sicherheit'
  };
}
if ('zh' == i18n.country) {
	Triology.groupmanager.I18n = {
		groupManagerText: '用户组管理',
		emptyText: '选择一个用户组.',
		removeTitleText: '删除用户组',
		removeMsgText: '删除用户组 "{0}"?',
		errorTitleText: '错误',
		errorMsgText: '用户组删除失败',
		colNameText: '名称',
		colDescriptionText: '描述',
		colMembersText: '成员',
		colGroupManagerText: '用户组管理',
		colCreationDateText: '创建日期',
		colTypeText: '类型',
		emptygroupManagerStoreText: '当前并没有配置任何用户组.',
		groupFormTitleText: '用户组表单',
		settingsText: '设置',
		updateErrorMsgText: '用户组更新失败',
		createErrorMsgText: '用户组创建失败',
		groupManagerHelpText: '源码库中的用户名.',
		groupManagerFormTitleText: '用户组管理表单',
		sectionSecurityText: '安全'
	};
}

loginCallbacks.push(function() {
  var navPanel = Ext.getCmp('navigationPanel');
  var securitySection = navPanel.items.get("securityConfig");

  var link = {
    label : Triology.groupmanager.I18n.groupManagerText,
    fn : Triology.groupmanager.addGroupManagerTabPanel,
    scope : this
  };

  // not XML users have no security section
  if (!securitySection){
    var count = navPanel.count() - 1;
    navPanel.insertSection(count, {
      id: 'securityConfig',
      title: Triology.groupmanager.I18n.sectionSecurityText,
      links: [link]
    });
  } else {
    securitySection.addLink(link);
    securitySection.tpl.overwrite(securitySection.body, {
      links: securitySection.links
    });
  }
});

Triology.groupmanager.addGroupManagerTabPanel = function() {
  main.addTabPanel('groupmanagerPanel', 'groupmanagerPanel', Triology.groupmanager.I18n.groupManagerText);
};

Triology.groupmanager.setEditPanel = function(panels) {
  var editPanel = Ext.getCmp('groupmanagerEditPanel');
  editPanel.removeAll();
  Ext.each(panels, function(panel) {
    editPanel.add(panel);
  });
  editPanel.setActiveTab(0);
  editPanel.doLayout();
};

/**
 * panels
 */
Triology.groupmanager.DefaultPanel = {
  region : 'south',
  title : Triology.groupmanager.I18n.groupManagerFormTitleText,
  xtype : 'panel',
  bodyCssClass : 'x-panel-mc',
  padding : 5,
  html : Triology.groupmanager.I18n.emptyText
};



Triology.groupmanager.Grid = Ext.extend(Sonia.rest.Grid, {

  colNameText : Triology.groupmanager.I18n.colNameText,
  colDescriptionText : Triology.groupmanager.I18n.colDescriptionText,
  colMembersText : Triology.groupmanager.I18n.colMembersText,
  colGroupManagerText : Triology.groupmanager.I18n.colGroupManagerText,
  colCreationDateText : Triology.groupmanager.I18n.colCreationDateText,
  colTypeText : Triology.groupmanager.I18n.colTypeText,
  emptygroupManagerStoreText : Triology.groupmanager.I18n.emptygroupManagerStoreText,
  groupFormTitleText : Triology.groupmanager.I18n.groupFormTitleText,

  // parent panel for history
  parentPanel : null,

  initComponent : function() {
    var groupManagerStore = new Sonia.rest.JsonStore({
      proxy : new Ext.data.HttpProxy({
        url : restUrl + 'groupmanager.json',
        disableCaching : false
      }),
      idProperty : 'name',
      fields : [ 'name', 'members', 'groupmanager', 'description', 'creationDate', 'type', 'properties' ],
      sortInfo : {
        field : 'name'
      }
    });

    var groupColModel = new Ext.grid.ColumnModel({
      defaults : {
        sortable : true,
        scope : this,
        width : 125
      },
      columns : [ {
        id : 'name',
        header : this.colNameText,
        dataIndex : 'name'
      }, {
        id : 'description',
        header : this.colDescriptionText,
        dataIndex : 'description',
        width : 300
      }, {
        id : 'members',
        header : this.colMembersText,
        dataIndex : 'members',
        renderer : this.renderMembers
      }, {
        id : 'groupmanager',
        header : this.colGroupManagerText,
        dataIndex : 'groupmanager',
        renderer : this.renderMembers
      }, {
        id : 'creationDate',
        header : this.colCreationDateText,
        dataIndex : 'creationDate',
        renderer : Ext.util.Format.formatTimestamp
      }, {
        id : 'type',
        header : this.colTypeText,
        dataIndex : 'type',
        width : 80
      } ]
    });

    var config = {
      autoExpandColumn : (admin ? 'groupmanager' : 'members'),
      store : groupManagerStore,
      colModel : groupColModel,
      emptyText : this.emptygroupManagerStoreText,
      listeners : {
        fallBelowMinHeight : {
          fn : this.onFallBelowMinHeight,
          scope : this
        }
      }
    };

    Ext.apply(this, Ext.apply(this.initialConfig, config));
    Triology.groupmanager.Grid.superclass.initComponent.apply(this, arguments);

    // store grid in parent panel
    this.parentPanel.groupmanagerGrid = this;
  },

  onFallBelowMinHeight : function(height, minHeight) {
    var p = Ext.getCmp('groupmanagerPanel');
    this.setHeight(minHeight);
    var epHeight = p.getHeight();
    p.setHeight(epHeight - (minHeight - height));
    // rerender
    this.doLayout();
    p.doLayout();
    this.ownerCt.doLayout();
  },

  renderMembers : function(members) {
    var out = '';
    if (members) {
      var s = members.length;
      for ( var i = 0; i < s; i++) {
        out += members[i];
        if ((i + 1) < s) {
          out += ', ';
        }
      }
    }
    return out;
  },

  selectItem : function(group) {
    if (debug) {
      console.debug(group.name + ' selected');
    }

    if (this.parentPanel) {
      this.parentPanel.updateHistory(group);
    }

    if (admin) {
      Triology.groupmanager.setEditPanel([ {
        item : group,
        xtype : 'groupManagerForm',
        listeners : {
          updated : {
            fn : this.reload,
            scope : this
          },
          created : {
            fn : this.reload,
            scope : this
          }
        }
      }, {
        item : group,
        xtype : 'groupManagerMemberForm',
        listeners : {
          updated : {
            fn : this.reload,
            scope : this
          },
          created : {
            fn : this.reload,
            scope : this
          }
        }
      } ]);
    } else {
      Triology.groupmanager.setEditPanel([ {
        item : group,
        xtype : 'groupManagerMemberForm',
        listeners : {
          updated : {
            fn : this.reload,
            scope : this
          },
          created : {
            fn : this.reload,
            scope : this
          }
        }
      } ]);
    }
  }

});

// register xtype
Ext.reg('groupmanagerGrid', Triology.groupmanager.Grid);


Triology.groupmanager.Panel = Ext.extend(Sonia.rest.Panel, {

  titleText : Triology.groupmanager.I18n.groupManagerFormTitleText,
  emptyText : Triology.groupmanager.I18n.emptyText,
  removeTitleText : Triology.groupmanager.I18n.removeTitleText,
  removeMsgText : Triology.groupmanager.I18n.removeMsgText,
  errorTitleText : Triology.groupmanager.I18n.errorTitleText,
  errorMsgText : Triology.groupmanager.I18n.errorMsgText,

  // grid
  groupmanagerGrid : null,

  initComponent : function() {
    var config = {
      tbar : [ {
        xtype : 'tbbutton',
        text : this.reloadText,
        icon : this.reloadIcon,
        scope : this,
        handler : this.reload
      } ],
      items : [ {
        id : 'groupmanagerGrid',
        xtype : 'groupmanagerGrid',
        region : 'center',
        parentPanel : this
      }, {
        id : 'groupmanagerEditPanel',
        xtype : 'tabpanel',
        activeTab : 0,
        height : 250,
        split : true,
        border : true,
        region : 'south',
        items : [ {
          bodyCssClass : 'x-panel-mc',
          title : this.titleText,
          padding : 5,
          html : this.emptyText
        } ]
      } ]
    };

    Ext.apply(this, Ext.apply(this.initialConfig, config));
    Sonia.group.Panel.superclass.initComponent.apply(this, arguments);
  },

  getGrid : function() {
    if (!this.groupmanagerGrid) {
      if (debug) {
        console.debug('could not find groupmanager grid, fetch by id');
      }
      this.groupmanagerGrid = Ext.getCmp('groupmanagerGrid');
    }
    return this.groupmanagerGrid;
  },

  updateHistory : function(group) {
    var token = Sonia.History.createToken('groupmanagerPanel', group.name);
    Sonia.History.add(token);
  },

  resetPanel : function() {
    Sonia.group.setEditPanel(Sonia.group.DefaultPanel);
  },

  reload : function() {
    this.getGrid().reload();
  }

});

// register xtype
Ext.reg('groupmanagerPanel', Triology.groupmanager.Panel);

// register history handler
Sonia.History.register('groupmanagerPanel', {

  onActivate : function(panel) {
    var token = null;
    var rec = panel.getGrid().getSelectionModel().getSelected();
    if (rec) {
      token = Sonia.History.createToken('groupmanagerPanel', rec.get('name'));
    } else {
      token = Sonia.History.createToken('groupmanagerPanel');
    }
    return token;
  },

  onChange : function(groupId) {
    var panel = Ext.getCmp('groupmanagerPanel');
    if (!panel) {
      Triology.groupmanager.addGroupManagerTabPanel();
      panel = Ext.getCmp('groupmanagerPanel');
      if (groupId) {
        var selected = false;
        panel.getGrid().getStore().addListener('load', function() {
          if (!selected) {
            panel.getGrid().selectById(groupId);
            selected = true;
          }
        });
      }
    } else {
      main.addTab(panel);
      if (groupId) {
        panel.getGrid().selectById(groupId);
      }
    }
  }
});


Triology.groupmanager.MemberFormPanel = Ext.extend(Sonia.group.MemberFormPanel, {
    
  update: function(group){
    if ( debug ){
      console.debug( 'update group ' + group.name );
    }
    group = Ext.apply( this.item, group );

    // this.updateMembers(group);
    this.fireEvent('preUpdate', group);

    var url = restUrl + 'groupmanager/group/' + group.name + '.json';
    var el = this.el;
    var tid = setTimeout( function(){el.mask('Loading ...');}, 100);

    Ext.Ajax.request({
      url: url,
      jsonData: group,
      method: 'PUT',
      scope: this,
      success: function(){
        if ( debug ){
          console.debug('update success');
        }
        this.fireEvent('updated', group);
        clearTimeout(tid);
        el.unmask();
        this.execCallback(this.onUpdate, group);
      },
      failure: function(result){
        this.fireEvent('updateFailed', group);
        clearTimeout(tid);
        el.unmask();
        main.handleRestFailure(
          result, 
          this.errorTitleText, 
          this.updateErrorMsgText
        );
      }
    });
  },
  
});

Ext.reg('groupManagerMemberForm', Triology.groupmanager.MemberFormPanel);


Triology.groupmanager.GroupManagerFormPanel = Ext.extend(Sonia.rest.FormPanel, {
  colMemberText : Triology.groupmanager.I18n.groupManagerText,
  membersText : Triology.groupmanager.I18n.groupManagerText,
  managerStore : null,
  addIcon : 'resources/images/add.png',
  removeIcon : 'resources/images/delete.png',
  errorTitleText : 'Error',
  updateErrorMsgText : 'Group-Manager update failed',
  createErrorMsgText : 'Group-Manager creation failed',
  
  // help
  nameHelpText : 'Unique name of the group.',
  descriptionHelpText : 'A short description of the group.',
  membersHelpText : 'Usernames of the group members.',
  

  initComponent : function() {
    this.addEvents('preCreate', 'created', 'preUpdate', 'updated', 'updateFailed', 'creationFailed');
    this.managerStore = new Ext.data.SimpleStore({
      fields : [ 'groupmanager' ],
      sortInfo : {
        field : 'groupmanager'
      }
    });

    var memberColModel = new Ext.grid.ColumnModel({
      defaults : {
        sortable : true
      },
      columns : [ {
        id : 'groupmanager',
        header : Triology.groupmanager.I18n.groupManagerText,
        dataIndex : 'groupmanager',
        editor : new Ext.form.ComboBox({
          store : userSearchStore,
          displayField : 'label',
          valueField : 'value',
          typeAhead : true,
          mode : 'remote',
          queryParam : 'query', // contents of the field sent to server.
          hideTrigger : true, // hide trigger so it doesn't look like a combobox.
          selectOnFocus : true,
          width : 250
        })
      } ]
    });

    var selectionModel = new Ext.grid.RowSelectionModel({
      singleSelect : true
    });

    if (this.item) {
      var data = [];
      if (this.item.groupmanager) {
        for ( var i = 0; i < this.item.groupmanager.length; i++) {
          var a = [];
          a.push(this.item.groupmanager[i]);
          data.push(a);
        }
      }
      this.managerStore.loadData(data);
    }

    var config = {
      title : this.membersText,
      listeners : {
        updated : this.clearModifications,
        preUpdate : {
          fn : this.updateGroupManager,
          scope : this
        }
      },
      items : [ {
        id : 'groupmanagerGrid',
        xtype : 'editorgrid',
        clicksToEdit : 1,
        frame : true,
        width : '100%',
        autoHeight : true,
        autoScroll : false,
        colModel : memberColModel,
        sm : selectionModel,
        store : this.managerStore,
        viewConfig : {
          forceFit : true
        },
        tbar : [ {
          text : this.addText,
          scope : this,
          icon : this.addIcon,
          handler : function() {
            var Member = this.managerStore.recordType;
            var grid = Ext.getCmp('groupmanagerGrid');
            grid.stopEditing();
            this.managerStore.insert(0, new Member());
            grid.startEditing(0, 0);
          }
        }, {
          text : this.removeText,
          scope : this,
          icon : this.removeIcon,
          handler : function() {
            var grid = Ext.getCmp('groupmanagerGrid');
            var selected = grid.getSelectionModel().getSelected();
            if (selected) {
              this.managerStore.remove(selected);
            }
          }
        }, '->', {
          id : 'groupmanagerGridHelp',
          xtype : 'box',
          autoEl : {
            tag : 'img',
            src : 'resources/images/help.png'
          }
        } ]
      } ]
    };

    Ext.apply(this, Ext.apply(this.initialConfig, config));
    Triology.groupmanager.GroupManagerFormPanel.superclass.initComponent.apply(this, arguments);
  },
  
  update : function(group) {
    if (debug) {
      console.debug('update group ' + group.name);
    }
    group = Ext.apply(this.item, group);

    this.fireEvent('preUpdate', group);

    var url = restUrl + 'groupmanager/' + group.name + '.json';
    var el = this.el;
    var tid = setTimeout(function() {
      el.mask('Loading ...');
    }, 100);

    Ext.Ajax.request({
      url : url,
      jsonData : group,
      method : 'PUT',
      scope : this,
      success : function() {
        if (debug) {
          console.debug('update success');
        }
        this.fireEvent('updated', group);
        clearTimeout(tid);
        el.unmask();
        this.execCallback(this.onUpdate, group);
      },
      failure : function(result) {
        this.fireEvent('updateFailed', group);
        clearTimeout(tid);
        el.unmask();
        main.handleRestFailure(result, this.errorTitleText, this.updateErrorMsgText);
      }
    });
  },

  create : function(item) {
    if (debug) {
      console.debug('create group: ' + item.name);
    }
    item.type = state.defaultUserType;

    var url = restUrl + 'groupmanager.json';
    var el = this.el;
    var tid = setTimeout(function() {
      el.mask('Loading ...');
    }, 100);

    this.fireEvent('preCreate', item);

    // this.updateMembers(item);

    Ext.Ajax.request({
      url : url,
      jsonData : item,
      method : 'POST',
      scope : this,
      success : function() {
        if (debug) {
          console.debug('create success');
        }
        this.fireEvent('created', item);
        // this.memberStore.removeAll();
        this.getForm().reset();
        clearTimeout(tid);
        el.unmask();
        this.execCallback(this.onCreate, item);
      },
      failure : function(result) {
        this.fireEvent('creationFailed', item);
        clearTimeout(tid);
        el.unmask();
        main.handleRestFailure(result, this.errorTitleText, this.createErrorMsgText);
      }
    });
  },

  cancel : function() {
    if (debug) {
      console.debug('cancel form');
    }
    Triology.groupmanager.setEditPanel(Triology.groupmanager.DefaultPanel);
  },

  afterRender : function() {
    // call super
    Triology.groupmanager.GroupManagerFormPanel.superclass.afterRender.apply(this, arguments);

    Ext.QuickTips.register({
      target : Ext.getCmp('groupmanagerGridHelp'),
      title : '',
      text : Triology.groupmanager.I18n.groupManagerHelpText,
      enabled : true
    });
  },

  updateGroupManager : function(item) {
    var managers = [];
    this.managerStore.data.each(function(record) {
      managers.push(record.data.groupmanager);
    });
    item.groupmanager = managers;
  },

  clearModifications : function() {
    Ext.getCmp('groupmanagerGrid').getStore().commitChanges();
  }

});

Ext.reg('groupManagerForm', Triology.groupmanager.GroupManagerFormPanel);

